%% R = gather(D, K, types, alpha)
% runs "process" on each run of the given data and collects the results.
%
% Input:
% D: the data structure as produced by gendata.
% K: the number of models: use polynomials of degree 0, ..., K-1.
% types: types(t) is true iff results should be obtained for the switch
%     distribution of type t-1:
%     - type 0 (required): fast switch distribution with block size exponent alpha
%     - type 1: slow switch distribution (reduced sample size)
%     - type 2: normal switch distribution
%
% Output:
% R: a structure with the following fields:
% R.types, R.alpha, R.K: the parameters of the experiment, for convenience
% R.coef{k}, R.XTX{k}: result of the order k fit for the entire sample
% R.M_sw<T>(n):     model selected by switch distribution of type T at sample size n
% R.M_bayes(n):     model selected by Bayes factors at sample size n
% R.M_aic(n):       model selected by AIC at sample size n
% R.M_bic(n):       model selected by BIC at sample size n
% R.L(k,1:n):       log(P_k(x^n))
% R.L_sw<T>(n):     log(P_switch(x^n)) for switch distribution of type T
% R.L_bayes:        log(P_BMA(x^n))
% R.L_aic:          log(P_AIC(x^n))
% R.L_bic:          log(P_BIC(x^n))
%
function R = gather(D, K, types, alpha)
  assert(types(1)==1);
  R.types = types; R.alpha = alpha; R.K = K;
  runs = size(D.X, 2);
  for run=1:runs
    fprintf('Run %d\n', run);
    fprintf('  T=0          '); tic; V = process(D.X(:,run),D.Y(:,run),D.N, K, 0, alpha); toc;
    if run==1, R.coef = V.coef;  R.A = V.A; R.L = V.L; end
    R.M_aic(run,:)   = V.M_aic;    R.L_aic(run,:)   = V.L_aic;
    R.M_bic(run,:)   = V.M_bic;    R.L_bic(run,:)   = V.L_bic;
    R.M_bayes(run,:) = V.M_bayes;  R.L_bayes(run,:) = V.L_bayes;
    R.M_sw0(run,:)   = V.M_switch; R.L_sw0(run,:)   = V.L_switch;
    if types(2)
      R.Nslow = ceil(1.75 .* D.N .^ (5./6));
      fprintf('  T=1 (N=%3d)  ', R.Nslow);
      tic; V = process(D.X(:,run),D.Y(:,run),R.Nslow, K, 1, alpha); toc;
      R.M_sw1(run,:) = V.M_switch; R.L_sw1(run,:) = V.L_switch;
    end
    if types(3)
      fprintf('  T=2          '); tic; V = process(D.X(:,run),D.Y(:,run),D.N, K, 2, alpha); toc;
      R.M_sw2(run,:) = V.M_switch; R.L_sw2(run,:) = V.L_switch;
    end
  end
end
